from django.db import models
from django.contrib.auth.models import AbstractUser
from django.core.validators import MinLengthValidator, RegexValidator, MaxLengthValidator
from django.core.exceptions import ValidationError
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _
from datetime import date
import re

def validate_email(value):
    if not re.match(r"[^@]+@[^@]+\.[^@]+", value):
        raise ValidationError("Email tidak valid")

def validate_url(value):
    if not re.match(r"^(?:http(s)?:\/\/)?[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:/?#[\]@!\$&'\(\)\*\+,;=.]+$", value):
        raise ValidationError("URL tidak valid")

def validate_phone(value):
    if not 8 <= len(value) <= 15:
        raise ValidationError("Nomor HP harus minimal 8 dan maksimal 15 digit.")
    
def validate_password_extra(value):
    if not re.search(r'[a-zA-Z]', value):
        raise ValidationError("Password harus mengandung setidaknya satu huruf.")
    if not re.search(r'\d', value):
        raise ValidationError("Password harus mengandung setidaknya satu angka.")
    if not re.search(r'[!@#$%^&*(),.?":{}|<>]', value):
        raise ValidationError("Password harus mengandung setidaknya satu karakter spesial.")
    if len(value) < 8:
        raise ValidationError("Password harus memiliki panjang minimal 8 karakter.")

class User(AbstractUser):
    username = models.CharField(
        _('username'),
        max_length=255,
        unique=True,
        validators=[
            RegexValidator(
                r'^[\w\.\-]+$',
                message="Username hanya boleh berisi huruf, angka, dan karakter ('.','_','-')."
            ),
        ],
        error_messages={
            'unique': _("Username ini sudah digunakan."),
        },
        blank=False,
        null=False,
        default='user',
    )
    email = models.EmailField(_('email address'), unique=True, validators=[validate_email], null=False, blank=False)

    password1 = models.CharField(
        max_length=255,
        validators=[validate_password_extra], default='hshsy01234567!'
    )
    
    tanggal_lahir = models.DateField(null=False, blank=False)
    nomor_hp = models.CharField(max_length=15, validators=[MinLengthValidator(8), RegexValidator(r'^[1-9]\d{1,3}\d{6,13}$', message="Format nomor HP tidak valid. Gunakan format tanpa simbol: [kode negara][nomor] dan tidak diawali dengan 0")], null=False, blank=False)
    url_blog = models.URLField(max_length=255, validators=[validate_url], null=True, blank=True)
    deskripsi_diri = models.TextField(validators=[MinLengthValidator(5), MaxLengthValidator(1000)],null=False, blank=False)
    kode_sku = models.CharField(max_length=9, validators=[MinLengthValidator(9), RegexValidator(r'[a-zA-Z]{3}-[0-9]{5}', message="Format kode SKU tidak valid, harus Prefiks 3 huruf bebas dan '-'. Kemudian ditambah 5 angka.")], null=False, blank=False, default='ABC-00000')
    id_transaksi = models.CharField(max_length=12, validators=[MinLengthValidator(12), RegexValidator(r'T-[0-9]{10}', message="Format ID transaksi tidak valid, harus Prefiks “T-” lalu ditambah 10 angka.")], null=False, blank=False, default='T-0000000000')

    def clean(self):
        if self.email:
            self.email = self.email.strip().lower()

        if self.tanggal_lahir:
            today = date.today()
            age = today.year - self.tanggal_lahir.year - ((today.month, today.day) < (self.tanggal_lahir.month, self.tanggal_lahir.day))
            if age < 12:
                raise ValidationError("Usia pengguna minimal 12 tahun.")

        super().clean() 
    
    def __str__(self):
        return self.username